#!
# -*- coding: utf-8 -*-

"""
╔════════════════════════════════════════════════════════════════════════════════════╗
║                                                                                    ║
║   Copyright (c) 2020 https://prrvchr.github.io                                     ║
║                                                                                    ║
║   Permission is hereby granted, free of charge, to any person obtaining            ║
║   a copy of this software and associated documentation files (the "Software"),     ║
║   to deal in the Software without restriction, including without limitation        ║
║   the rights to use, copy, modify, merge, publish, distribute, sublicense,         ║
║   and/or sell copies of the Software, and to permit persons to whom the Software   ║
║   is furnished to do so, subject to the following conditions:                      ║
║                                                                                    ║
║   The above copyright notice and this permission notice shall be included in       ║
║   all copies or substantial portions of the Software.                              ║
║                                                                                    ║
║   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,                  ║
║   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES                  ║
║   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.        ║
║   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY             ║
║   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,             ║
║   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE       ║
║   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                    ║
║                                                                                    ║
╚════════════════════════════════════════════════════════════════════════════════════╝
"""

import unohelper

import traceback


class OptionsView(unohelper.Base):
    def __init__(self, window, index, timeout, exist):
        self._window = window
        self.setSynchronizePolicy(index)
        self.setTimeout(timeout)
        self._getDatasource().Model.Enabled = exist

# OptionsView getter methods
    def getSynchronizePolicy(self):
        return self._getOptionIndex()

    def getTimeout(self):
        return int(self._getTimeout().Value)

# OptionsView setter methods
    def setSynchronizePolicy(self, index):
        self._getOption(index).State = 1
        self.enableTimeout(index != 3)

    def setTimeout(self, timeout):
        self._getTimeout().Value = timeout

    def enableTimeout(self, enabled):
        self._getTimeoutLabel().Model.Enabled = enabled
        self._getTimeout().Model.Enabled = enabled

# OptionsView private methods
    def _getOptionIndex(self):
        for index in range(1,4):
            if self._getOption(index).State:
                return index

# OptionsView private control methods
    def _getOption(self, index):
        return self._window.getControl('OptionButton%s' % index)

    def _getTimeoutLabel(self):
        return self._window.getControl('Label2')

    def _getTimeout(self):
        return self._window.getControl('NumericField1')

    def _getDatasource(self):
        return self._window.getControl('CommandButton1')

